/***************************************************
  Adafruit MQTT Library FONA Example

  Designed specifically to work with the Adafruit FONA
  ----> http://www.adafruit.com/products/1946
  ----> http://www.adafruit.com/products/1963
  ----> http://www.adafruit.com/products/2468
  ----> http://www.adafruit.com/products/2542

  These cellular modules use TTL Serial to communicate, 2 pins are
  required to interface.

  Adafruit invests time and resources providing this open source code,
  please support Adafruit and open-source hardware by purchasing
  products from Adafruit!

  Written by Limor Fried/Ladyada for Adafruit Industries.
  MIT license, all text above must be included in any redistribution
  
  Modified by JFUENTES 2019
  
  Tested with SIM800L board connected to a SAMD21-MINI and itsy bitsy M0 board
  
 ****************************************************/
#include <Adafruit_SleepyDog.h>
#include "Adafruit_FONA.h"
#include "Adafruit_MQTT.h"
#include "Adafruit_MQTT_FONA.h"

/************ Serial 2 ************/
#include <Arduino.h>   // required before wiring_private.h
#include "wiring_private.h" // pinPeripheral() function
 
Uart Serial2 (&sercom1, 11, 10, SERCOM_RX_PAD_0, UART_TX_PAD_2);

void SERCOM1_Handler()
{
  Serial2.IrqHandler();
}

/*************************** FONA Pins ***********************************/
#define FONA_RST 9
Adafruit_FONA fona = Adafruit_FONA(FONA_RST);

/*************************** Variables ***********************************/
String newLine = ""; //gets the line asked to the main micro board
char *cstr = new char[50];

/************************* WiFi Access Point *********************************/

  // Optionally configure a GPRS APN, username, and password.
  // You might need to do this to access your network's GPRS/data
  // network.  Contact your provider for the exact APN, username,
  // and password values.  Username and password are optional and
  // can be removed, but APN is required.
#define FONA_APN       ""
#define FONA_USERNAME  ""
#define FONA_PASSWORD  ""

/************************* Adafruit.io Setup *********************************/

#define AIO_SERVER      "broker.mqttdashboard.com"
#define AIO_SERVERPORT  1883
#define AIO_USERNAME    ""
#define AIO_KEY         ""

/************ Global State (you don't need to change this!) ******************/

// Setup the FONA MQTT class by passing in the FONA class and MQTT server and login details.
Adafruit_MQTT_FONA mqtt(&fona, AIO_SERVER, AIO_SERVERPORT, AIO_USERNAME, AIO_KEY);

// You don't need to change anything below this line!
#define halt(s) { Serial.println(F( s )); while(1);  }

// FONAconnect is a helper function that sets up the FONA and connects to
// the GPRS network. See the fonahelper.cpp tab above for the source!
boolean FONAconnect(const __FlashStringHelper *apn, const __FlashStringHelper *username, const __FlashStringHelper *password);

/****************************** Feeds ***************************************/

// Setup a feed called ' ' for publishing.
// Notice MQTT paths for AIO follow the form: <username>/feeds/<feedname>
Adafruit_MQTT_Publish Temp = Adafruit_MQTT_Publish(&mqtt, AIO_USERNAME "SPART_tx/"); //Topic

// Setup a feed called ' ' for subscribing to changes.
//Adafruit_MQTT_Subscribe onoffbutton = Adafruit_MQTT_Subscribe(&mqtt, AIO_USERNAME "testtopic/T_P_T");

/*************************** Sketch Code ************************************/

// How many transmission failures in a row we're willing to be ok with before reset
uint8_t txfailures = 0;
#define MAXTXFAILURES 3

void setup() {
   
  //Watchdog is optional!
  Watchdog.enable(30000);
  
  Serial.begin(9600); //USB - SerialUSB when we use SAMD21-MINI (and code needs to be changed acordingly)  
  Serial1.begin(4800); //Modem - Connected to the modem
  Serial2.begin(9600); //Connected to the main drifter
  
//while (!Serial);

  // Serial2 conf. Assign pins 10 & 11 SERCOM functionality
  pinPeripheral(10, PIO_SERCOM);
  pinPeripheral(11, PIO_SERCOM);
  
  //GPS.begin(9600);
  Serial.println(F("Adafruit FONA MQTT demo"));

  //mqtt.subscribe(&onoffbutton);

  Watchdog.reset();
  delay(5000);  // wait a few seconds to stabilize connection
  Watchdog.reset();
  
  Serial.println("1");
  
  // Initialise the FONA module
  while (! FONAconnect(F(FONA_APN), F(FONA_USERNAME), F(FONA_PASSWORD))) {
    Serial.println("Retrying FONA");
  }
  Serial.println("2");

  Serial.println(F("Connected to Cellular!"));

  Watchdog.reset();
  delay(5000);  // wait a few seconds to stabilize connection
  Watchdog.reset();
  
  MQTT_connect();
  
  pinMode(12, OUTPUT);

}

uint32_t x=0;

void loop() {
  
  Watchdog.reset(); //I have to start it...
  MQTT_connect();
   
  TakeLine(); //asks for a line to the main drifter board
      
  strcpy(cstr, newLine.c_str());
  
  digitalWrite(13, LOW);
  Temp.publish(cstr);
  digitalWrite(13, HIGH);
  
  Serial.println(newLine);
  
}


void TakeLine() //Serial comunication between Microcontrolers
{
  newLine=""; //empty newLine string
  Serial2.write("l");

   while (Serial2.available()) {
    delay(1);  //small delay to allow input buffer to fill
    char c = Serial2.read();  //gets one byte from serial buffer
    if (c == 'f') {
      break; //breaks out of capture loop to print readstring
    }  
    newLine += c; //makes the string readString
  }
}

// Function to connect and reconnect as necessary to the MQTT server.
// Should be called in the loop function and it will take care if connecting.
void MQTT_connect() {
  int8_t ret;

  // Stop if already connected.
  if (mqtt.connected()) {
    return;
  }
  
  Serial.print("Connecting to MQTT... ");

  while ((ret = mqtt.connect()) != 0) { // connect will return 0 for connected
    Serial.println(mqtt.connectErrorString(ret));
    Serial.println("Retrying MQTT connection in 5 seconds...");
    mqtt.disconnect();
    delay(5000);  // wait 5 seconds
  }
  Serial.println("MQTT Connected!");
}
